// Defines for app.read_time
#define READ_TIME_INDEFINITELY			0xFE		// Read time indefinitely

// Defines for app.buzzer_options
#define BUZZER_ENABLED					0x01
#define STARTUP_BUZZER					0x02
#define BUZZER_TONE_SINGLE				0x00
#define BUZZER_TONE_HIGH_LOW			0x10
#define BUZZER_TONE_LOW_HIGH			0x20
#define BUZZER_TONE_MASK				0x30

// Defines for app.output_date_time_format
#define TIME_FORMAT_HH_MM_SS			0x00
#define TIME_FORMAT_HH_MM				0x01
#define TIME_FORMAT_MASK				0x03

#define DATE_FORMAT_DASH_DD_MM_YYYY		0x00
#define DATE_FORMAT_DASH_MM_DD_YYYY		0x10
#define DATE_FORMAT_DASH_DD_MM_YY		0x20
#define DATE_FORMAT_DASH_MM_DD_YY		0x30
#define DATE_FORMAT_DASH_YYYY_MM_DD		0x40
#define DATE_FORMAT_DASH_YY_MM_DD		0x50
#define DATE_FORMAT_DASH_DD_MM			0x60
#define DATE_FORMAT_DASH_MM_DD			0x70
#define DATE_FORMAT_SLASH_DD_MM_YYYY	0x80
#define DATE_FORMAT_SLASH_MM_DD_YYYY	0x90
#define DATE_FORMAT_SLASH_DD_MM_YY		0xA0
#define DATE_FORMAT_SLASH_MM_DD_YY		0xB0
#define DATE_FORMAT_SLASH_YYYY_MM_DD	0xC0
#define DATE_FORMAT_SLASH_YY_MM_DD		0xD0
#define DATE_FORMAT_SLASH_DD_MM			0xE0
#define DATE_FORMAT_SLASH_MM_DD			0xF0
#define DATE_FORMAT_MASK				0xF0

// Defines for app.output_quantity_options
#define QUANTITY_DELETE_KEY_DISABLED	0x01
#define QUANTITY_DELETE_ZERO_QNTY		0x04
#define QUANTITY_ALLOW_NEGATIVE_QNTY	0x08

// Defines for app.output_field_separator
#define FIELD_SEPARATOR_ZERO			0x00
#define FIELD_SEPARATOR_TAB				0x09
#define FIELD_SEPARATOR_LF				0x0A
#define FIELD_SEPARATOR_CR				0x0D
#define FIELD_SEPARATOR_GS				0x1D
#define FIELD_SEPARATOR_COMMA			','
#define FIELD_SEPARATOR_SEMICOLON		';'
#define FIELD_SEPARATOR_SLASH			'/'
#define FIELD_SEPARATOR_PIPE			'|'

// Defines for app.output_sequence
#define SEQUENCE_NONE					0x00
#define SEQUENCE_BARCODE				0x01
#define SEQUENCE_DATE					0x02
#define SEQUENCE_TIME					0x03
#define SEQUENCE_SERIAL_NUMBER			0x04
#define SEQUENCE_QUANTITY				0x05
#define SEQUENCE_MAX					SEQUENCE_QUANTITY

//
// public functions
//
extern void Application_Default(void);
extern void Version_Output(void);
extern void UsbSetVcp(void);
extern int UsbSetMsd(void);

//
// Defines a structure of configuration settings that is stored in non-volatile (flash) memory
// to make it possible that certain configurations aren't lost after a restart
//
typedef struct
{
	unsigned char default_interface;						// Interface: USB-VCP/Cradle or USB-MSD
	unsigned char read_mode;								// Read mode
	unsigned char read_time;								// Read time (stored in 0.1s steps)
	unsigned char goodread_duration;						// Good read led duration (stored in 20ms steps)
	unsigned char buzzer_volume;							// Buzzer volume
	unsigned char buzzer_duration;							// Buzzer duration (stored in 40ms steps)
	unsigned char buzzer_options;							// Buzzer tone
	unsigned char special_options;							// RFU: Special options (Reserved for future use)

	unsigned char output_field_separator;
	unsigned char output_date_time_format;
	unsigned char output_barcode_size;
	unsigned char output_quantity_size;
	unsigned char output_quantity_options;
	unsigned char output_serial_enabled;
	unsigned char output_sequence[5];
} application_vars;

extern application_vars app;			// Structure containing all application settings that will be stored in flash memory
