//
// The following table implements systemsetting options that need to be added to the OS or overruled
//
// Note: The use of 'systemsetting()' inside the functions in the table below is limited to a protect against recursion,
//       It is still possible to use systemsetting() but only with a 1 menu option per function call.
//
const OPTION menu_option_table[] =
{                                               // CLEAR mask                              SET mask (use 0xFF 0xFF in case of a function call)
	{ "GC",(void*)&(app.buzzer_options),				0x00,                                   STARTUP_BUZZER              },
	{ "GD",(void*)&(app.buzzer_options),				STARTUP_BUZZER,                         0x00                        },
	{ "S0",(void*)&(app.read_mode),						0xFF,                                   SINGLE                      },
	{ "S1",(void*)&(app.read_mode),						0xFF,                                   MULTIPLE                    },
	{ "S2",(void*)&(app.read_mode),						0xFF,                                   CONTINUOUS                  },
	{ "T0",(void*)&(app.buzzer_volume),					0xFF,                                   VHIGH                       },
	{ "T1",(void*)&(app.buzzer_volume),					0xFF,                                   VMEDIUM                     },
	{ "T2",(void*)&(app.buzzer_volume),					0xFF,                                   VSTANDARD                   },
	{ "T3",(void*)&(app.buzzer_volume),					0xFF,                                   VLOW                        },
	{ "T4",(void*)&(app.goodread_duration),				0xFF,                                   0                           },
	{ "T5",(void*)&(app.goodread_duration),				0xFF,                                   10                          },	// 200ms
	{ "T6",(void*)&(app.goodread_duration),				0xFF,                                   20                          },	// 400ms
	{ "T7",(void*)&(app.goodread_duration),				0xFF,                                   40                          },	// 800ms
    { "U2",(void*)Application_Default,					0xFF,                                   0xFF                        },
	{ "W0",(void*)&(app.buzzer_options),				BUZZER_ENABLED,                         0x00                        },
	{ "W1",(void*)&(app.buzzer_options),				BUZZER_TONE_MASK,                       BUZZER_TONE_SINGLE          },
	{ "W2",(void*)&(app.buzzer_options),				BUZZER_TONE_MASK,                       BUZZER_TONE_HIGH_LOW        },
	{ "W3",(void*)&(app.buzzer_options),				BUZZER_TONE_MASK,                       BUZZER_TONE_LOW_HIGH        },
	{ "W4",(void*)&(app.buzzer_duration),				0xFF,                                   TSHORT                      },  // 100ms (=actually 120ms)
	{ "W5",(void*)&(app.buzzer_duration),				0xFF,                                   TSTANDARD                   },  // 200ms
	{ "W6",(void*)&(app.buzzer_duration),				0xFF,                                   TLONG                       },  // 400ms
	{ "W7",(void*)&(app.buzzer_duration),				0xFF,                                   TCLICK                      },  // 50ms (=actually 40ms)
	{ "W8",(void*)&(app.buzzer_options),				0x00,                                   BUZZER_ENABLED              },
	{ "Y0",(void*)&(app.read_time),						0xFF,                                   0                           },
	{ "Y1",(void*)&(app.read_time),						0xFF,                                   10                          },
	{ "Y2",(void*)&(app.read_time),						0xFF,                                   20                          },
	{ "Y3",(void*)&(app.read_time),						0xFF,                                   30                          },
	{ "Y4",(void*)&(app.read_time),						0xFF,                                   40                          },
	{ "Y5",(void*)&(app.read_time),						0xFF,                                   50                          },
	{ "Y6",(void*)&(app.read_time),						0xFF,                                   60                          },
	{ "Y7",(void*)&(app.read_time),						0xFF,                                   70                          },
	{ "Y8",(void*)&(app.read_time),						0xFF,                                   80                          },
	{ "Y9",(void*)&(app.read_time),						0xFF,                                   90                          },
	{ "YM",(void*)&(app.read_time),						0xFF,                                   READ_TIME_INDEFINITELY      },
    { "Z1",(void*)Version_Output,						0xFF,                                   0xFF                        },
    { "C01",(void*)UsbSetVcp,							0xFF,                                   0xFF                        },
    { "C04",(void*)UsbSetMsd,							0xFF,                                   0xFF                        },

	// The following menu options are non-official OSE menu options, which have been added for output file formatting
	{ "+D0",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_DASH_DD_MM_YYYY	},
	{ "+D1",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_DASH_MM_DD_YYYY },	
	{ "+D2",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_DASH_DD_MM_YY   },	
	{ "+D3",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_DASH_MM_DD_YY   },	
	{ "+D4",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_DASH_YYYY_MM_DD	},
	{ "+D5",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_DASH_YY_MM_DD	},
	{ "+D6",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_DASH_DD_MM		},
	{ "+D7",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_DASH_MM_DD		},		
	{ "+D8",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_SLASH_DD_MM_YYYY},
	{ "+D9",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_SLASH_MM_DD_YYYY},
	{ "+DA",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_SLASH_DD_MM_YY	},
	{ "+DB",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_SLASH_MM_DD_YY	},
	{ "+DC",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_SLASH_YYYY_MM_DD},
	{ "+DD",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_SLASH_YY_MM_DD	},
	{ "+DE",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_SLASH_DD_MM		},
	{ "+DF",(void*)&(app.output_date_time_format),		DATE_FORMAT_MASK,						DATE_FORMAT_SLASH_MM_DD		},
    { "+T0",(void*)&(app.output_date_time_format),		TIME_FORMAT_MASK,						TIME_FORMAT_HH_MM_SS        },
    { "+T1",(void*)&(app.output_date_time_format),		TIME_FORMAT_MASK,						TIME_FORMAT_HH_MM           },
	{ "+FC",(void*)&(app.output_field_separator),		0xFF,									FIELD_SEPARATOR_COMMA		},
	{ "+FG",(void*)&(app.output_field_separator),		0xFF,									FIELD_SEPARATOR_GS			},
	{ "+FH",(void*)&(app.output_field_separator),		0xFF,									FIELD_SEPARATOR_SEMICOLON	},
	{ "+FL",(void*)&(app.output_field_separator),		0xFF,									FIELD_SEPARATOR_LF			},
	{ "+FP",(void*)&(app.output_field_separator),		0xFF,									FIELD_SEPARATOR_PIPE		},
	{ "+FR",(void*)&(app.output_field_separator),		0xFF,									FIELD_SEPARATOR_CR			},
	{ "+FS",(void*)&(app.output_field_separator),		0xFF,									FIELD_SEPARATOR_SLASH		},
	{ "+FT",(void*)&(app.output_field_separator),		0xFF,									FIELD_SEPARATOR_TAB			},
	{ "+FZ",(void*)&(app.output_field_separator),		0xFF,									FIELD_SEPARATOR_ZERO		},
	{ "+Q1",(void*)&(app.output_quantity_size),			0xFF,									0x01						},
	{ "+Q2",(void*)&(app.output_quantity_size),			0xFF,									0x02						},
	{ "+Q3",(void*)&(app.output_quantity_size),			0xFF,									0x03						},
	{ "+Q4",(void*)&(app.output_quantity_size),			0xFF,									0x04						},
	{ "+Q5",(void*)&(app.output_quantity_size),			0xFF,									0x05						},
	{ "+Q6",(void*)&(app.output_quantity_size),			0xFF,									0x06						},
	{ "+Q7",(void*)&(app.output_quantity_size),			0xFF,									0x07						},
	{ "+Q8",(void*)&(app.output_quantity_size),			0xFF,									0x08						},
	{ "+Q9",(void*)&(app.output_quantity_size),			0xFF,									0x09						},
	{ "+QB",(void*)&(app.output_quantity_options),		0x00,									QUANTITY_DELETE_KEY_DISABLED},    
	{ "+QC",(void*)&(app.output_quantity_options),		QUANTITY_DELETE_KEY_DISABLED,			0x00						},
	{ "+QN",(void*)&(app.output_quantity_options),		0x00,									QUANTITY_ALLOW_NEGATIVE_QNTY},
	{ "+QP",(void*)&(app.output_quantity_options),		QUANTITY_ALLOW_NEGATIVE_QNTY,			0x00						},
	{ "+QY",(void*)&(app.output_quantity_options),		QUANTITY_DELETE_ZERO_QNTY,				0x00						},
	{ "+QZ",(void*)&(app.output_quantity_options),		0x00,									QUANTITY_DELETE_ZERO_QNTY	},
	{ "+B00",(void*)&(app.output_barcode_size),			0x0F,									0x00						},
	{ "+B01",(void*)&(app.output_barcode_size),			0x0F,									0x01						},
	{ "+B02",(void*)&(app.output_barcode_size),			0x0F,									0x02						},
	{ "+B03",(void*)&(app.output_barcode_size),			0x0F,									0x03						},
	{ "+B04",(void*)&(app.output_barcode_size),			0x0F,									0x04						},
	{ "+B05",(void*)&(app.output_barcode_size),			0x0F,									0x05						},
	{ "+B06",(void*)&(app.output_barcode_size),			0x0F,									0x06						},
	{ "+B07",(void*)&(app.output_barcode_size),			0x0F,									0x07						},
	{ "+B08",(void*)&(app.output_barcode_size),			0x0F,									0x08						},
	{ "+B09",(void*)&(app.output_barcode_size),			0x0F,									0x09						},
	{ "+B0-",(void*)&(app.output_barcode_size),			0xF0,									0x00						},
	{ "+B10",(void*)&(app.output_barcode_size),			0xF0,									0x10						},
	{ "+B20",(void*)&(app.output_barcode_size),			0xF0,									0x20						},
	{ "+B30",(void*)&(app.output_barcode_size),			0xF0,									0x30						},
	{ "+B40",(void*)&(app.output_barcode_size),			0xF0,									0x40						},
	{ "+B50",(void*)&(app.output_barcode_size),			0xF0,									0x50						},
	{ "+B60",(void*)&(app.output_barcode_size),			0xF0,									0x60						},
	{ "+B70",(void*)&(app.output_barcode_size),			0xF0,									0x70						},
	{ "+B80",(void*)&(app.output_barcode_size),			0xF0,									0x80						},
	{ "+B90",(void*)&(app.output_barcode_size),			0xF0,									0x90						},
	{ "+S0B",(void*)&(app.output_sequence[0]),			0xFF,									SEQUENCE_BARCODE			},
	{ "+S0D",(void*)&(app.output_sequence[0]),			0xFF,									SEQUENCE_DATE				},
	{ "+S0T",(void*)&(app.output_sequence[0]),			0xFF,									SEQUENCE_TIME				},
	{ "+S0S",(void*)&(app.output_sequence[0]),			0xFF,									SEQUENCE_SERIAL_NUMBER		},
	{ "+S0Q",(void*)&(app.output_sequence[0]),			0xFF,									SEQUENCE_QUANTITY			},
	{ "+S1N",(void*)&(app.output_sequence[1]),			0xFF,									SEQUENCE_NONE				},
	{ "+S1B",(void*)&(app.output_sequence[1]),			0xFF,									SEQUENCE_BARCODE			},
	{ "+S1D",(void*)&(app.output_sequence[1]),			0xFF,									SEQUENCE_DATE				},
	{ "+S1T",(void*)&(app.output_sequence[1]),			0xFF,									SEQUENCE_TIME				},
	{ "+S1S",(void*)&(app.output_sequence[1]),			0xFF,									SEQUENCE_SERIAL_NUMBER		},
	{ "+S1Q",(void*)&(app.output_sequence[1]),			0xFF,									SEQUENCE_QUANTITY			},
	{ "+S2N",(void*)&(app.output_sequence[2]),			0xFF,									SEQUENCE_NONE				},
	{ "+S2B",(void*)&(app.output_sequence[2]),			0xFF,									SEQUENCE_BARCODE			},
	{ "+S2D",(void*)&(app.output_sequence[2]),			0xFF,									SEQUENCE_DATE				},
	{ "+S2T",(void*)&(app.output_sequence[2]),			0xFF,									SEQUENCE_TIME				},
	{ "+S2S",(void*)&(app.output_sequence[2]),			0xFF,									SEQUENCE_SERIAL_NUMBER		},
	{ "+S2Q",(void*)&(app.output_sequence[2]),			0xFF,									SEQUENCE_QUANTITY			},
	{ "+S3N",(void*)&(app.output_sequence[3]),			0xFF,									SEQUENCE_NONE				},
	{ "+S3B",(void*)&(app.output_sequence[3]),			0xFF,									SEQUENCE_BARCODE			},
	{ "+S3D",(void*)&(app.output_sequence[3]),			0xFF,									SEQUENCE_DATE				},
	{ "+S3T",(void*)&(app.output_sequence[3]),			0xFF,									SEQUENCE_TIME				},
	{ "+S3S",(void*)&(app.output_sequence[3]),			0xFF,									SEQUENCE_SERIAL_NUMBER		},
	{ "+S3Q",(void*)&(app.output_sequence[3]),			0xFF,									SEQUENCE_QUANTITY			},
	{ "+S4N",(void*)&(app.output_sequence[4]),			0xFF,									SEQUENCE_NONE				},
	{ "+S4B",(void*)&(app.output_sequence[4]),			0xFF,									SEQUENCE_BARCODE			},
	{ "+S4D",(void*)&(app.output_sequence[4]),			0xFF,									SEQUENCE_DATE				},
	{ "+S4T",(void*)&(app.output_sequence[4]),			0xFF,									SEQUENCE_TIME				},
	{ "+S4S",(void*)&(app.output_sequence[4]),			0xFF,									SEQUENCE_SERIAL_NUMBER		},
	{ "+S4Q",(void*)&(app.output_sequence[4]),			0xFF,									SEQUENCE_QUANTITY			},
};

#define MAX_OPTIONS sizeof(menu_option_table)/sizeof(OPTION)
