//------------------------------------------------------------------
//  Main header file of Bluetooth Demo application for the OPN2002
//------------------------------------------------------------------

// Defines and variables used for barcode reading
#define	MAX_BARCODE_LENGTH				2700

// Defines for app.blt_interface
#define BLT_VCP_DEVICE					0x00
#define BLT_HID_DEVICE					0x01

// Defines for app.trigger_to_connect_options
#define TRIGGER_TO_CONNECTABLE			0x01		// If set, trigger to connect acts as 'trigger to make connectable'

// Defines for app.connect_to
#define CONNECT_TO_MASK					0x03
#define CONNECT_TO_PC					0x00
#define CONNECT_TO_CRADLE				0x01
#define CONNECT_TO_CRADLE_HID			0x02

// Defines for app.memorizing
#define MEMORIZING_ENABLED				0x01
#define MEMORIZING_AFTER_CONNECT_LOSS	0x00
#define MEMORIZING_WHEN_NOT_CONNECTED	0x04

// Defines for app.read_time
#define READ_TIME_INDEFINITELY			0xFE		// Read time indefinitely

// Defines for app.buzzer_options
#define BUZZER_ENABLED					0x01
#define STARTUP_BUZZER					0x02
#define BUZZER_TONE_SINGLE				0x00
#define BUZZER_TONE_HIGH_LOW			0x10
#define BUZZER_TONE_LOW_HIGH			0x20
#define BUZZER_TONE_MASK				0x30

// Defines for app.serial_options
#define RS_CONFIG_DISABLED   			0x01
#define RS_BUZZER_DISABLED   			0x02
#define RS_TRIGGER_DISABLED  			0x04
#define RS_GR_LED_DISABLED   			0x08
#define RS_ACK_NAK_ON		 			0x10
#define RS_LASER_DISABLED    			0x20
#define RS_MOTOR_DISABLED    			0x80

// Defines for app.ser_cmd_format
#define CMD_TERM_CR    					0x01      // Serial command terminator: CR
#define CMD_TERM_LF    					0x02      // Serial command terminator: LF
#define CMD_TERM_ETX   					0x04      // Serial command terminator: ETX
#define CMD_TERM_EOT   					0x08      // Serial command terminator: EOT
#define CMD_TERM_MASK  					0x0F
#define CMD_HEADER_ESC 					0x10      // Serial command header: ESC
#define CMD_HEADER_STX 					0x20      // Serial command header: STX
#define CMD_HEADER_MASK					0x30

// Defines for app.options 
#define ENABLE_KEYBOARD_TOGGLE			0x01
#define ENABLE_WAKEUP_KEY				0x02
#define USB_HID_MODE					0x04
#define USB_VCP_MODE					0x08

// Verification string used by 'LoadApplicationSettings' to if any possible application settings
// found in flash memory are compatible with this application. If this version check fails only change this version if the application settings of the previous version aren't
// all application settings will be set back to default
#define	VERSION_CHECK					"BLT_DEMOv1.4"		
															
//
// Defines a structure of configuration settings that is stored in non-volatile (flash) memory
// to make it possible that certain configurations aren't lost after a restart
//
typedef struct
{
	unsigned char blt_interface;							// Indicates the currently selected interface (HID or VCP)
	unsigned char blt_hid_manual_pin;						// If TRUE, the PIN code must be entered using numeric direct input labels (HID only, Menu book chapter 4.3.3)
	unsigned char trigger_to_connect_time;					// Trigger to connect time (stored in seconds, 0 = disabled)
	unsigned char trigger_to_connectable_time;				// Trigger to make connectable time (stored in seconds, 0 = disabled)
	unsigned char trigger_to_disconnect_time;				// Trigger to disconnect time (stored in seconds, 0 = disabled)
	unsigned char reconnect_attempts;						// Number of reconnect attempts after connection loss (0 = disabled)
	unsigned char connectable_time;							// Time that the device stays connectable (stored in 10s steps)
	unsigned char read_mode;								// Read mode
	unsigned char read_time;								// Read time (stored in 0,1s steps)
	unsigned char goodread_duration;						// Good read led duration (stored in 20ms steps)
	unsigned char buzzer_volume;							// Buzzer volume
	unsigned char buzzer_duration;							// Buzzer duration (stored in 40ms steps)
	unsigned char buzzer_options;							// Buzzer tone
	unsigned char ser_cmd_options;							// Serial command options (OSE serial commands allowed)
	unsigned char ser_cmd_format;							// Serial command format options
	unsigned char options;									// Additional options, for now only used for iOS 'enable keyboeard toggle flag'
} application_vars;

extern application_vars app;

