/***********************************************************************
 *
 *  Copyright (C) Opticon Sensors Europe B.V.
 *
 ***********************************************************************/

#ifndef __LIB_H__
#define __LIB_H__

 /*
 *--------------------------------------
 *  General definitions
 *--------------------------------------
 */
#define OK                  0
#define ERROR               -1
#define ON                  1
#define OFF                 0
#define YES                 1
#define NO                  0
#define TRUE                1
#define FALSE               0
#define DISABLE             0
#define ENABLE				1
#define NG					100
#define STANDBY				2
#define	DISABLED			-2
#define RESET               3
#define NOW                 5


/*
 *--------------------------------------
 *  ASCII definitions
 *--------------------------------------
 */
#define SOH             0x01
#define STX             0x02
#define ETX             0x03
#define EOT             0x04
#define ENQ             0x05
#define ACK             0x06
#define BEL             0x07
#define BS              0x08
#define HT              0x09
#define LF              0x0A
#define VT              0x0B
#define FF              0x0C
#define CR              0x0D
#define SO              0x0E
#define SI              0x0F
#define DLE             0x10
#define DC1             0x11
#define DC2             0x12
#define DC3             0x13
#define DC4             0x14
#define NAK             0x15
#define SYN             0x16
#define ETB             0x17
#define CAN_CHAR        0x18
#define EM              0x19
#define SUB             0x1A
#define ESC             0x1B
#define FS              0x1C
#define GS              0x1D
#define RS              0x1E
#define US              0x1F
#define SP              0x20
#define DEL             0x7F

/*
 *--------------------------------------
 *  Physical keyboard defines
 *--------------------------------------
 */
#define TRIGGER_KEY			0x80
#define	ENT_KEY				TRIGGER_KEY

#define UP_KEY				0x81
#define DOWN_KEY			UP_KEY
#define	ESC_KEY				UP_KEY
#define	DELETE_KEY			UP_KEY
#define CLR_KEY				UP_KEY


/*
 *--------------------------------------
 *  USB / Bluetooth HID Keyboard defines
 *--------------------------------------
 */
#define	KB_BS_KEY				0xB0
#define	KB_TAB_KEY				0xB1
#define	KB_ENT_KEY				0xB2
#define	KB_ESC_KEY				0xB3
#define	KB_DOWN_KEY				0xB4
#define	KB_UP_KEY				0xF3
#define	KB_RIGHT_KEY			0xB6
#define	KB_LEFT_KEY				0xB7
#define	KB_DELETE_KEY			0xB8
#define	KB_INSERT_KEY			0xB9
#define	KB_HOME_KEY				0xBA
#define	KB_END_KEY				0xBB
#define	KB_PAGE_UP_KEY			0xBC
#define	KB_PAGE_DOWN_KEY		0xFB
#define	KB_CAPSLOCK_KEY			0xBE

#define	NUMPAD_0_KEY			0xBF
#define	NUMPAD_1_KEY			0xC0
#define	NUMPAD_2_KEY			0xC1
#define	NUMPAD_3_KEY			0xC2
#define	NUMPAD_4_KEY			0xC3
#define	NUMPAD_5_KEY			0xC4
#define	NUMPAD_6_KEY			0xC5
#define	NUMPAD_7_KEY			0xC6
#define	NUMPAD_8_KEY			0xC7
#define	NUMPAD_9_KEY			0xC8

#define	KB_F1_KEY				0xF0
#define	KB_F2_KEY				0xE2
#define	KB_F3_KEY				0xC9
#define	KB_F4_KEY				0xCA
#define	KB_F5_KEY				0xCB
#define	KB_F6_KEY				0xCC
#define	KB_F7_KEY				0xCD
#define	KB_F8_KEY				0xCE
#define	KB_F9_KEY				0xF2
#define	KB_F10_KEY				0xD0
#define	KB_F11_KEY				0xF1
#define	KB_F12_KEY				0xD2

#define	NUMLOCK_KEY				0xD3
#define	NUMPAD_SLASH_KEY		0xD4
#define	NUMPAD_STAR_KEY			0xD5
#define	NUMPAD_MINUS_KEY		0xA9
#define	NUMPAD_PLUS_KEY			0xD7
#define	NUMPAD_ENTER_KEY		0xD8

#define	LEFT_SHIFT_KEY			0xD9
#define	RIGHT_SHIFT_KEY			0xDA
#define	LEFT_CONTROL_KEY		0xDB
#define	RIGHT_CONTROL_KEY		0xDC
#define	LEFT_ALT_KEY			0x9F
#define	RIGHT_ALT_KEY			0xDE
#define	LEFT_GUI_KEY			0xDF
#define	WINDOWS_KEY				LEFT_GUI_KEY
#define	RIGHT_GUI_KEY			0xFC
#define	MENU_KEY				0xE8

#define	NUMPAD_POINT_KEY		0xE3
#define	PRINT_SCREEN_KEY		0xE4
#define	SCROLL_LOCK_KEY			0xE5
#define	PAUSE_KEY				0xFE
#define	EURO_SIGN				0xE7

#define TOGGLE_KEYBOARD			0xA6		// Toggle iPhone/iPad onscreen keyboard

/*
 *--------------------------------------
 *  scnanner functions
 *--------------------------------------
 */
/* bar code ID member */
/*
 * Constants for id member
 */
#define EAN13           0x01
#define UPCA            0x02
#define EAN8            0x03
#define UPCE            0x04
#define UPCE1			0x05
#define ISBN			0x06
#define ISSN			0x07
#define ISMN			0x08
#define CODE39          0x09
#define CODABAR         0x0A
#define NW7             CODABAR
#define D2OF5           0x0B                    // industrial 2 of 5
#define I2OF5           0x0C                    // interleaved 2 of 5
#define CODE93          0x0D
#define CODE128         0x0E
#define MSI_PLESSEY     0x0F
#define IATA            0x10
#define UK_PLESSEY      0x11
#define TELEPEN         0x12
#define M2OF5           0x13
#define CHINPOST        0x14
#define NW7_ABC         0x15
#define NW7_CX          0x16
#define SCODE           0x17
#define LAETUS          0x18
#define TRIOPTIC        LAETUS
#define C39_FA          0x19                    // Full ascii
#define C39_IT          0x1A                    // Italian pharma code
#define EAN128          0x1B
#define RSS_14          0x1C
#define RSS_LIMITED     0x1D
#define RSS_EXPANDED    0x1E
#define PDF417          0x1F
#define CCC_COMPOSITE   0x20
#define MICRO_PDF417    0x21
#define CCA_COMPOSITE   0x22
#define CCB_COMPOSITE   0x23
#define CODE11          0x24
#define EAN128_CCC      0x25                    // EAN-128 with CC-C composite
#define CODE3OF5        0x26                    // Code 3 of 5 A.K.A. Korean Postal Code
#define CODABLOCK_F		0x27

/*** ADD ON +2 ***/
#define ADD_ON_2        0x40
#define EAN13_2         0x41
#define UPCA_2          0x42
#define EAN8_2          0x43
#define UPCE_2          0x44
#define UPCE1_2         0x45
#define CODE128_2       0x4E                    // the same group CODE-128

/*** ADD ON +5 ***/
#define ADD_ON_5        0x80
#define EAN13_5         0x81
#define UPCA_5          0x82
#define EAN8_5          0x83
#define UPCE_5          0x84
#define UPCE1_5         0x85
#define CODE128_5       0x8E                    // the same group CODE-128

#define ADD_ON_MASK     0xC0
#define MENU_CODE       0xF0                    // OSE Code-39 Menu label
#define MENU_CODE_PDF   0xF1                    // OSE (u)PDF417 Menu label
#define MENU_CODE_C128  0xF2                    // OSE Code-128 Menu label

/* ScannerPower */
#define SINGLE              2
#define MULTIPLE            4
#define CONTINUOUS          6
#define TRIGGER             8

/*
 *--------------------------------------
 *  Led/sound functions
 *--------------------------------------
 */
/* GoodRead */
#define RED					0x01
#define GREEN				0x02
#define BLUE				0x04			// Not supported by the OPN2003
#define ORANGE				(RED|GREEN)
#define RED_FLASH			0x10
#define GREEN_FLASH			0x20
#define ORANGE_FLASH		(RED_FLASH|GREEN_FLASH)
#define BLUE_FLASH			0x40			// Not supported by the OPN2003

#define FOREVER             -1

/*
 * definitions for sound volume level
 */
#define VOFF				0
#define VVLOW				1
#define VLOW				2
#define VSTANDARD			3
#define VMEDIUM				4
#define VHIGH				5

 /*
 * definitions for sound tone duration
 */
#define TCLICK              1
#define TSHORT              3
#define TSTANDARD           5
#define TLONG               10
#define TVLONG              20

/*
 * definitions for sound tone
 */
#define SPAUSE              1
#define SERROR              100
#define SLOW                1000
#define SMEDIUM             2500
#define SHIGH               3500

/* ischarging */
#define OFF_CRADLE          0
#define CHARGING            1
#define CHARGING_FULL       2
#define CHARGING_DISABLED   3           /* connected to power supply, butnot able to charge for various reasons */
										/* (i.e. temperature is out of range, charging process is initializing) */

/* UsbStatus */
#define USB_NOT_CONNECTED	0
#define USB_POWERED	        1
#define USB_OPENED			3

/*
 * Return values for 'ExecuteMenuLabel()'
 */
#define	ENTERING_MENU_MODE 		1
#define	EXITING_MENU_MODE		2
#define	INVALID_OPTION_READ		3
#define VALID_OPTION_READ		4
#define LABEL_IGNORED			5

/*
 *--------------------------------------
 *  Memory/file functions
 *--------------------------------------
 */
/*  file system */
#define D_NAME              8
#define D_EXT               3
#define MAX_FNAME			13

#define ERR_NOT_SUPPORTED   (-2)
#define ERR_PARAMETER       (-3)
#define ERR_NOT_INSERTED    (-4)
#define ERR_NOT_FORMAT      (-5)

#ifndef O_BINARY
	#define O_BINARY        0x00    /*   binary file                 */
#endif

/*
 *--------------------------------------
 * Serial communication functions
 *--------------------------------------
 */
#define COM0                0		// USB-VCP (when connected to USB),  Cradle (when in cradle)
#define COM1                COM0
#define COM2                COM0
#define COM3                3		// Bluetooth SPP/Master
#define COM4				4		//
#define COM5                5		// Bluetooth SPP/Slave

#define COM9				9		// USB VCP (using COM0 is highly recommended to allow automatic downloading of software to stay operational)
#define COM10				10		// USB HID
#define COM11				11		// USB MSD

#define	COM12               12		// Bluetooth HID Slave
#define	COM13               13		// Bluetooth HID Master

/*
 * definitions for xmodem receive
 */
#define XMR_OK              0
#define XMR_CREATE_ERR      1
#define XMR_POLL_ERR        2
#define XMR_ABORT_ERR       3
#define XMR_BLOCK_ERR       4
#define XMR_CRC_ERR         5
#define XMR_CHECKSUM_ERR    6
#define XMR_WRITE_ERR       7
#define XMR_RESPONSE_ERR    8
#define XMR_MEM_ERR         9

/*
 * definitions for xmodem transmit
 */
#define XMT_OK              0
#define XMT_OPEN_ERR        1
#define XMT_RESPONSE_ERR    2
#define XMT_POLL_ERR        3
#define XMT_READ_ERR        4
#define XMT_ACK_ERR1        5
#define XMT_ACK_ERR2        6
#define XMT_MEM_ERR         7

#define ERR_USER_BREAK      -3
#define ERR_EOT_HOST        -5
#define ERR_HOST_REQ        -6
#define ERR_OPEN_FILE       -8
#define ERR_WRITE_FILE      -9
#define ERR_READ_FILE       -10
#define ERR_OUT_SYNC        -12
#define ERR_ALOC_MEM        -13
#define ERR_BCC_FAIL        -2          // checksum error (internal)
#define ERR_TIME_OUT        -4          // time out error (internal)
#define ERR_HOST_LOST       -7          // time out on first char (internal)
#define ERR_NAK_RECV        -11         // NAK received from host (internal)


/*
 ***********************************************************************
 *  Type Define
 ***********************************************************************
 */
struct barcode
{
    char *text;
    int length;
    int id;
    int min;
    int max;
};

struct time
{
    unsigned char ti_hour;
    unsigned char ti_min;
    unsigned char ti_sec;
};

struct date
{
    unsigned int  da_year;
    unsigned char da_day;
    unsigned char da_mon;
};

struct ffblk
{
    char name[D_NAME+1];
    char ext[D_EXT+1];
    int filesize;
    char search_name[D_NAME+1];
    char search_ext[D_EXT+1];
    int slot;
    int drive;
};

typedef const struct
{
    char name[4];
    void* variable;
    unsigned char clear_mask;
    unsigned char set_mask;
}OPTION;

//-------------------------------------------------------------------
//---------------------- Prototypes ---------------------------------
//-------------------------------------------------------------------
//            Public Operating System functions
//-------------------------------------------------------------------

// Barcode / laser functions
void scannerpower(int mode, int time);
int readbarcode(struct barcode *barcodep);
int scanneroff(void);
void ResetReadTime(void);
int IsInstoreBarcode(void);
int IsAimingOn(void);
void AimingOn(void);
void AimingOff(void);

// Configuration function(s)
int systemsetting(const char *string);
int ExecuteMenuLabel(struct barcode *code);
int SystemSettingsMemorizing(int on_off);
int ApplicationSettingsMemorizing(int on_off, void *application_settings, int size, char *match_string);
int AddMenuOptionTable(OPTION menu_table[], int nr_of_entries);


// Battery & temperature functions
unsigned int batteryvoltage(void);
unsigned int GetBatteryVoltage(void);
unsigned int GetBatteryVoltage_mV(void);
int ischarging(void);
unsigned int GetTemperature(void);
int	IsBatteryLow(void);

// Date & time functions
void getdate(struct date *dp);
void gettime(struct time *tp);
int checktime(struct time *timep);
int checkdate(struct date *datep);
int setdate(struct date *dp);
int settime(struct time *timep);
void SetAlarm(struct time *timep, unsigned short onoff);

// Timing functions
void delay(int time);
void starttimer(unsigned int timevalue);
unsigned int endtimer(void);
unsigned int timervalue(void);
void SetSystemTime(unsigned long);
unsigned long GetSystemTime(void);
void SetTickCount(unsigned int value);
unsigned int GetTickCount(void);

// LED / Sound functions
void goodreadled(int on_off, int time);
int isBuzzerOn(void);
void sound(int time, int vol, ...);

// Key functions
unsigned int triggerpressed(void);
unsigned int uppressed(void);
unsigned int kbhit(void);
unsigned int downpressed(void);				// = uppressed() on OPN200x
void resetkey(void);
void WaitKey(void);
int ClearKeyAsTrigger(int enabled);
int StandDetectionPressed(void);			// For OPR3101 and OPR3301 only
int AutoTriggerPressed(void);				// For OPR3101 and OPR3301 only

// File transfer functions
int neto_receive(char (*filename)[MAX_FNAME], const char *term_id, int tkey, int ypos);
int neto_receive2(char (*filename)[MAX_FNAME], const char *term_id, unsigned int timeout, int tkey, int ypos);
int neto_transmit(char (*filename)[MAX_FNAME], int file_count, const char *term_id, int tkey, int ypos);
int neto_transmit2(char (*filename)[MAX_FNAME], int file_count, const char *term_id, unsigned int timeout, int tkey, int ypos);
int xmodemreceive(const char *filename);
int xmodemtransmit(const char *filename);

// Version & identification functions
void version_os(char os[10]);
void version_rom(char rom[10]);
char *GetOsVersion(void);
char *GetBootVersion(void);
unsigned int getterminalid(void);
void setterminalid(unsigned int id);
long GetSerialNumber(void);

// Communication functions
int comopen(unsigned int port);
void comclose(unsigned int port);
int getcom(int timeout);
int putcom(int data);
void resetreceive(void);
void setRTS(int on_off);
void setDTR(int on_off);
int getCTS(void);
int getDSR(void);

// USB functions
int UsbStatus(void);

// Power management functions
void idle(void);
int powereddown(void);
void autopowerdown(int what, unsigned long time);

// File and memory functions
unsigned long coreleft(void);			// Returns memory left on the file system
unsigned long Heap_CoreLeft(void);		// Returns memory left on Heap
int findnext(struct ffblk *ffblk);
int findfirst(const char *fname,struct ffblk *ffblk);
int format(void);
long fsize(const char *filename);
long lseek(int handle, long offset, int orgin);
long tell(int handle);
int chsize(int handle, long size);
#define far_malloc malloc						// void *far_malloc (unsigned long nbytes);

// Download / Installation functions
void download(void);
void downloadapplication(int port);
int runapplfromfile(char *filename);


//
// Special functions that are not mentioned in the manual
//

// Flash functions
int GetFlashSize(void);
void eraseblock(int block_nr);
void erasesector(int sector_nr);
void programbyte(unsigned char *address, unsigned char d);
void programword(unsigned short data, unsigned short *address);
void programbuffer(unsigned char *src, unsigned char *dest, unsigned int len);

// Interrupts
void DisableInts(void);
void EnableInts(void);

// Heartbeat
typedef void(*heartbeat_function)(void);
void InstallHeartbeatHandler( heartbeat_function userfunction );
void RemoveHeartbeatHandler( heartbeat_function userfunction );

// Output functions
void PutByte(unsigned char c);
void PutShort(unsigned short c);
void PutLong(unsigned long l);
int putstring(char *data);
int putnstring(const char *data, int length);

// Reset
void SoftReset(void);

// Bluetooth function
unsigned char *GetBltAddress(void);
unsigned char *GetBltPin(void);
void SetBltAddress(const char *address);
void SetBltPin(const char *pincode);
unsigned char *GetBltLocalName( void );
void SetBltLocalName(const char *localname );
char *GetBltLocalAddress(void);

int	blt_is_connected( void );

typedef struct
{
	char			bd_address[13];
	char			name[20];
	unsigned long 	address_l;
	unsigned long 	address_h;
	unsigned char 	page_scan_rpt_mode;
	unsigned char 	page_scan_period_mode;
	unsigned short 	clock_offset;
}device_info;

int blt_discovery( device_info *dev_inf, int abortkey );
void blt_unpair(void);

typedef void (*pin_callback) (char* pin, int* len  );
void blt_set_pin_callback(pin_callback cb );

/*
 *--------------------------------------------
 * Display defines and functions
 *
 * (dummies that are added to stay compatible
 *  with other Opticon handheld terminals
 *  that do have a display)
 *--------------------------------------------
 */
#define DISP_WIDTH  	128
#define DISP_HEIGHT 	128

/* setpixel / drawline */
#define INVERT              2

/* setfont */
#define TINY_FONT           0x01
#define SMALL_FONT          0x02
#define MEDIUM_FONT         0x03
#define LARGE_FONT          0x04

/* cursor */
#define BLINKING            2
#define AUTOWRAP            3
#define NOWRAP              4

/* backlight */
#define TOGGLE              2
#define AUTO                4
#define SUSPEND             16
#define RESUME              32

void gotoxy(int x, int y)	__attribute__ ((deprecated));
int wherex(void)			__attribute__ ((deprecated));
int wherey(void)			__attribute__ ((deprecated));
int setecho(int echomode)	__attribute__ ((deprecated));
int contrast(int value)		__attribute__ ((deprecated));
void cursor(int mode)		__attribute__ ((deprecated));

void printsymbol(unsigned int data)							__attribute__ ((deprecated));
int backlight(int what, short time)							__attribute__ ((deprecated));
void drawline(int x1,int y1,int x2,int y2,int mode)			__attribute__ ((deprecated));
void getfontsize(unsigned int *height, unsigned int *width) __attribute__ ((deprecated));
int setfont(unsigned int type, unsigned char *font_ptr)		__attribute__ ((deprecated));
void setpixel(int x, int y, int state)						__attribute__ ((deprecated));

#endif		// __LIB_H__
